/*
 *   Copyright (c) 2022 Robin E. R. Davies
 *   All rights reserved.

 *   Permission is hereby granted, free of charge, to any person obtaining a copy
 *   of this software and associated documentation files (the "Software"), to deal
 *   in the Software without restriction, including without limitation the rights
 *   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions:

 *   The above copyright notice and this permission notice shall be included in all
 *   copies or substantial portions of the Software.

 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *   SOFTWARE.
 */

#include "InterpolatingDelay.hpp"

using namespace LsNumerics;

InterpolatingDelay::InterpolatingDelay()
{
}
InterpolatingDelay::InterpolatingDelay(uint32_t maxDelay)
{
    SetMaxDelay(maxDelay);
}

void InterpolatingDelay::SetMaxDelay(uint32_t maxDelay)
{
    uint32_t size = LsNumerics::NextPowerOfTwo(maxDelay + 2); // +one for the interpolator, +one to prevent wrapping.
    delayLine.resize(size);
    indexMask = size - 1;
    Clear();
}

void InterpolatingDelay::Clear()
{
    delayIndex = 0;
    for (size_t i = 0; i < delayLine.size(); ++i)
    {
        delayLine[i] = 0;
    }
}
